// Copyright 2025, IPng Networks GmbH, Pim van Pelt <pim@ipng.ch>

package vpp

import (
	"flag"

	"go.fd.io/govpp/adapter/socketclient"
	"go.fd.io/govpp/adapter/statsclient"
	"go.fd.io/govpp/api"
	"go.fd.io/govpp/binapi/vpe"
	"go.fd.io/govpp/core"

	"govpp-snmp-agentx/logger"
)

var (
	// Flags for VPP configuration
	ApiAddr       = flag.String("vppstats.api.addr", "/var/run/vpp/api.sock", "VPP API socket path")
	StatsAddr     = flag.String("vppstats.stats.addr", "/var/run/vpp/stats.sock", "VPP stats socket path")
	IfIndexOffset = flag.Int("vppstats.ifindex-offset", 1000, "Offset to add to VPP interface indices for SNMP")
	Period        = flag.Int("vppstats.period", 10, "Interval in seconds for querying VPP interface stats")
)

// VPPClient manages VPP connections and provides a unified interface
type VPPClient struct {
	apiConn   *core.Connection
	statsConn *core.StatsConnection
	connected bool
}

// NewVPPClient creates a new VPP client instance
func NewVPPClient() *VPPClient {
	return &VPPClient{}
}

// Connect establishes connections to both VPP API and Stats sockets
func (c *VPPClient) Connect() error {
	logger.Debugf("Connecting to VPP (API: %s, Stats: %s)", *ApiAddr, *StatsAddr)

	// Connect to API socket
	apiConn, err := core.Connect(socketclient.NewVppClient(*ApiAddr))
	if err != nil {
		return err
	}

	// Connect to stats socket
	statsClient := statsclient.NewStatsClient(*StatsAddr)
	statsConn, err := core.ConnectStats(statsClient)
	if err != nil {
		// Clean up API connection on stats failure
		func() {
			defer func() {
				if r := recover(); r != nil {
					logger.Debugf("Recovered from API disconnect during stats error: %v", r)
				}
			}()
			apiConn.Disconnect()
		}()
		return err
	}

	c.apiConn = apiConn
	c.statsConn = statsConn
	c.connected = true

	logger.Printf("Connected to VPP (API: %s, Stats: %s)", *ApiAddr, *StatsAddr)
	return nil
}

// Disconnect closes all VPP connections safely
func (c *VPPClient) Disconnect() {
	if c.apiConn != nil {
		func() {
			defer func() {
				if r := recover(); r != nil {
					logger.Debugf("Recovered from API disconnect panic: %v", r)
				}
			}()
			c.apiConn.Disconnect()
		}()
		c.apiConn = nil
	}

	if c.statsConn != nil {
		func() {
			defer func() {
				if r := recover(); r != nil {
					logger.Debugf("Recovered from stats disconnect panic: %v", r)
				}
			}()
			c.statsConn.Disconnect()
		}()
		c.statsConn = nil
	}

	c.connected = false
}

// IsConnected returns true if both API and Stats connections are active
func (c *VPPClient) IsConnected() bool {
	return c.connected && c.apiConn != nil && c.statsConn != nil
}

// GetAPIConnection returns the API connection for direct use
func (c *VPPClient) GetAPIConnection() *core.Connection {
	return c.apiConn
}

// GetStatsConnection returns the stats connection for direct use
func (c *VPPClient) GetStatsConnection() *core.StatsConnection {
	return c.statsConn
}

// NewAPIChannel creates a new API channel from the connection
func (c *VPPClient) NewAPIChannel() (api.Channel, error) {
	if c.apiConn == nil {
		return nil, &VPPError{Message: "API connection not established"}
	}
	return c.apiConn.NewAPIChannel()
}

// CheckLiveness performs a VPP liveness check using ShowVersion API call
func (c *VPPClient) CheckLiveness() bool {
	if !c.IsConnected() {
		return false
	}

	ch, err := c.NewAPIChannel()
	if err != nil {
		logger.Debugf("Failed to create API channel for liveness check: %v", err)
		return false
	}

	var channelClosed bool
	defer func() {
		if !channelClosed {
			defer func() {
				if r := recover(); r != nil {
					logger.Debugf("Recovered from channel close panic: %v", r)
				}
			}()
			ch.Close()
		}
	}()

	req := &vpe.ShowVersion{}
	reply := &vpe.ShowVersionReply{}

	if err := ch.SendRequest(req).ReceiveReply(reply); err != nil {
		logger.Debugf("VPP ShowVersion failed: %v", err)
		func() {
			defer func() {
				if r := recover(); r != nil {
					logger.Debugf("Channel already closed during error handling")
				}
			}()
			ch.Close()
			channelClosed = true
		}()
		return false
	}

	ch.Close()
	channelClosed = true

	logger.Debugf("VPP liveness check passed (version: %s)", string(reply.Version))
	return true
}

// VPPError represents a VPP-specific error
type VPPError struct {
	Message string
}

func (e *VPPError) Error() string {
	return e.Message
}
