// Copyright 2025, IPng Networks GmbH, Pim van Pelt <pim@ipng.ch>

package vpp

import (
	"testing"
)

func TestNewVPPClient(t *testing.T) {
	client := NewVPPClient()

	if client == nil {
		t.Fatal("NewVPPClient() returned nil")
	}

	if client.IsConnected() {
		t.Error("NewVPPClient() should return disconnected client")
	}

	if client.GetAPIConnection() != nil {
		t.Error("NewVPPClient() should have nil API connection initially")
	}

	if client.GetStatsConnection() != nil {
		t.Error("NewVPPClient() should have nil stats connection initially")
	}
}

func TestVPPClientDisconnect(t *testing.T) {
	client := NewVPPClient()

	// Should be safe to call disconnect on unconnected client
	client.Disconnect()

	if client.IsConnected() {
		t.Error("Client should not be connected after Disconnect()")
	}
}

func TestVPPClientNewAPIChannelWithoutConnection(t *testing.T) {
	client := NewVPPClient()

	_, err := client.NewAPIChannel()
	if err == nil {
		t.Error("NewAPIChannel() should return error when not connected")
	}

	vppErr, ok := err.(*VPPError)
	if !ok {
		t.Errorf("Expected VPPError, got %T", err)
	}

	if vppErr.Message != "API connection not established" {
		t.Errorf("Expected specific error message, got: %s", vppErr.Message)
	}
}

func TestVPPClientCheckLivenessWithoutConnection(t *testing.T) {
	client := NewVPPClient()

	if client.CheckLiveness() {
		t.Error("CheckLiveness() should return false when not connected")
	}
}

func TestVPPError(t *testing.T) {
	err := &VPPError{Message: "test error"}

	if err.Error() != "test error" {
		t.Errorf("VPPError.Error() returned %q, expected %q", err.Error(), "test error")
	}
}

func TestVPPClientConnectWithInvalidPaths(t *testing.T) {
	// Save original values
	origApiAddr := *ApiAddr
	origStatsAddr := *StatsAddr

	// Set invalid paths
	*ApiAddr = "/tmp/nonexistent_api.sock"
	*StatsAddr = "/tmp/nonexistent_stats.sock"

	// Restore original values after test
	defer func() {
		*ApiAddr = origApiAddr
		*StatsAddr = origStatsAddr
	}()

	client := NewVPPClient()
	err := client.Connect()

	if err == nil {
		t.Error("Connect() should fail with invalid socket paths")
		client.Disconnect() // Clean up if somehow it connected
	}

	if client.IsConnected() {
		t.Error("Client should not be connected after failed Connect()")
	}
}
